#!/usr/bin/env python3

import argparse
import requests
import time
import sys
import configparser
from suntime import Sun

parser = argparse.ArgumentParser()
parser.add_argument("-s",
                  "--status",
                  action="store_true",
                  dest="printStatus",
                  help="Print Status Information",
                  default=False)
parser.add_argument("-f",
                  "--force",
                  action="store_true",
                  dest="force",
                  help="Force retrieval of Power value",
                  default=False)
args = parser.parse_args()


def isDaylight(lat, lon, toleranceSeconds):
    daylight = False
    sun = Sun(lat, lon)
    sunriseTimestamp = int(sun.get_local_sunrise_time().timestamp())
    sunsetTimestamp = int(sun.get_local_sunset_time().timestamp())
    nowTimestamp = int(time.time())

    if ((sunriseTimestamp + toleranceSeconds) < nowTimestamp) and (
        (sunsetTimestamp - toleranceSeconds) > nowTimestamp):
        daylight = True

    return daylight


def getCurrentPower(userName, password, stationId):
    with requests.Session() as s:
        try:
            r = s.post('https://www.envertecportal.com/apiaccount/login',
                       data={
                           'userName': userName,
                           'pwd': password
                       },
                       timeout=(20, 40))

            r = s.post(
                'https://www.envertecportal.com/ApiStations/getStationInfo',
                data={
                    'stationId': stationId
                },
                timeout=(20, 60)).json()
            power = r['Data']['Power']

            r = s.post('https://www.envertecportal.com/apiAccount/Logout',
                       timeout=(20, 40))

        # connect timeouts occur so frequently since the portal relaunch,
        # ignore them for the time beeing completely
        except requests.exceptions.ConnectTimeout as eTimeout:
            sys.exit(1)
        except requests.exceptions.RequestException as e:
            print(e)
            raise SystemExit(e)

    return float(power)


# use our stateFile to determine if we have a state change
# used to decide if we print something - thus generate a mail - later on
def stateCheck(newState, stateFile):
    try:
        with open(stateFile, "r") as f:
            oldState = f.read()
    except FileNotFoundError:
        oldState = "NULL"

    if newState == oldState:
        change = False
    else:
        change = True
        with open(stateFile, "w") as f:
            f.write(newState)

    return change


# read configuration file
conf = configparser.ConfigParser()
conf.read('portalmonitor.ini')

# retrieve current power value as reported by envertecportal
if isDaylight(conf['config'].getfloat('lat'), conf['config'].getfloat('lon'),
              conf['config'].getint('toleranceSeconds')) or args.force:
    currentPower = getCurrentPower(conf['config']['userName'],
                                   conf['config']['password'],
                                   conf['config']['stationId'])

    if args.printStatus:
        print(f"Current Power: {currentPower}")

    if currentPower == 0:
        if stateCheck('FAILED', conf['config']['stateFile']):
            print('Error: Power dropped to 0 but we should have daylight!')
        sys.exit(1)
    else:
        if stateCheck('OK', conf['config']['stateFile']):
            print(f"Resolved: Inverter reports {currentPower}W")
